@@ -95,7 +95,7 @@ int q_main(int argc, char **argv)
 
 	if (install) {
 		char buf[_Q_PATH_MAX];
-		const char *prog;
+		const char *prog, *dir;
 		ssize_t rret;
 		int fd, ret;
 
@@ -104,20 +104,16 @@ int q_main(int argc, char **argv)
 
 		rret = readlink("/proc/self/exe", buf, sizeof(buf) - 1);
 		if (rret == -1) {
-			char *ptr = which("q");
-			if (ptr == NULL) {
-				warnfp("haha no symlink love for you");
-				return 1;
-			}
-			strncpy(buf, ptr, sizeof(buf));
-			buf[sizeof(buf) - 1] = '\0';
-		} else
-			buf[rret] = '\0';
+			warnfp("haha no symlink love for you");
+			return 1;
+		}
+		buf[rret] = '\0';
 
 		prog = basename(buf);
-		fd = open(dirname(buf), O_RDONLY|O_CLOEXEC);
+		dir = dirname(buf);
+		fd = open(dir, O_RDONLY|O_CLOEXEC|O_PATH);
 		if (fd < 0) {
-			warnfp("chdir(%s) failed", buf);
+			warnfp("open(%s) failed", dir);
 			return 1;
 		}
 
