@@ -1,10 +1,9 @@
 /*
- * Copyright 2005-2010 Gentoo Foundation
+ * Copyright 2005-2014 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/qfile.c,v 1.66 2013/04/29 05:32:43 vapier Exp $
  *
  * Copyright 2005-2010 Ned Ludd        - <solar@gentoo.org>
- * Copyright 2005-2010 Mike Frysinger  - <vapier@gentoo.org>
+ * Copyright 2005-2014 Mike Frysinger  - <vapier@gentoo.org>
  */
 
 #ifdef APPLET_qfile
@@ -36,7 +35,6 @@ static const char * const qfile_opts_help[] = {
 	"Exact match (used with --exclude)",
 	COMMON_OPTS_HELP
 };
-static const char qfile_rcsid[] = "$Id: qfile.c,v 1.66 2013/04/29 05:32:43 vapier Exp $";
 #define qfile_usage(ret) usage(ret, QFILE_FLAGS, qfile_long_opts, qfile_opts_help, lookup_applet_idx("qfile"))
 
 #define qfile_is_prefix(path, prefix, prefix_length) \
@@ -116,7 +114,7 @@ _q_static int qfile_cb(q_vdb_pkg_ctx *pkg_ctx, void *priv)
 		}
 		if (state->exclude_slot == NULL)
 			goto qlist_done; /* "(CAT/)?(PN|PF)" matches, and no SLOT specified */
-		eat_file_at(pkg_ctx->fd, "SLOT", state->buf, state->buflen);
+		q_vdb_pkg_eat(pkg_ctx, "SLOT", &state->buf, &state->buflen);
 		rmspace(state->buf);
 		if (strcmp(state->exclude_slot, state->buf) == 0)
 			goto qlist_done; /* "(CAT/)?(PN|PF):SLOT" matches */
@@ -217,8 +215,11 @@ _q_static int qfile_cb(q_vdb_pkg_ctx *pkg_ctx, void *priv)
 					}
 				}
 				if (state->slotted) {
-					eat_file_at(pkg_ctx->fd, "SLOT", slot+1, sizeof(slot)-1);
-					rmspace(slot+1);
+					/* XXX: This assumes the buf is big enough. */
+					char *slot_hack = slot + 1;
+					size_t slot_len = sizeof(slot) - 1;
+					q_vdb_pkg_eat(pkg_ctx, "SLOT", &slot_hack, &slot_len);
+					rmspace(slot_hack);
 					slot[0] = ':';
 				} else
 					slot[0] = '\0';
