@@ -26,7 +26,7 @@
 # include <sys/sysctl.h>
 #endif
 
-#define QLOP_DEFAULT_LOGFILE CONFIG_EPREFIX "var/log/emerge.log"
+#define QLOP_DEFAULT_LOGFILE "emerge.log"
 
 #define QLOP_FLAGS "gtHluscf:" COMMON_FLAGS
 static struct option const qlop_long_opts[] = {
@@ -48,7 +48,7 @@ static const char * const qlop_opts_help[] = {
 	"Show unmerge history",
 	"Show sync history",
 	"Show current emerging packages",
-	"Read emerge logfile instead of " QLOP_DEFAULT_LOGFILE,
+	"Read emerge logfile instead of $EMERGE_LOG_DIR/" QLOP_DEFAULT_LOGFILE,
 	COMMON_OPTS_HELP
 };
 #define qlop_usage(ret) usage(ret, QLOP_FLAGS, qlop_long_opts, qlop_opts_help, lookup_applet_idx("qlop"))
@@ -208,7 +208,7 @@ show_merge_times(char *package, const char *logfile, int average, char human_rea
 							if (human_readable)
 								print_seconds_for_earthlings(t[1] - t[0]);
 							else
-								printf("%s%"PRIu64"u%s seconds", GREEN, (uint64_t)(t[1] - t[0]), NORM);
+								printf("%s%"PRIu64"%s seconds", GREEN, (uint64_t)(t[1] - t[0]), NORM);
 							puts("");
 						}
 						merge_time += (t[1] - t[0]);
@@ -304,12 +304,31 @@ show_emerge_history(char listflag, int argc, char **argv, const char *logfile)
 	fclose(fp);
 }
 
+/* The format of the sync log has changed over time.
+
+Old format:
+1106804103: Started emerge on: Jan 27, 2005 05:35:03
+1106804103:  *** emerge  sync
+1106804103:  === sync
+1106804103: >>> starting rsync with rsync://192.168.0.5/gentoo-portage
+1106804537: === Sync completed with rsync://192.168.0.5/gentoo-portage
+1106804538:  *** terminating.
+
+New format:
+1431764402: Started emerge on: May 16, 2015 04:20:01
+1431764402:  *** emerge --quiet --keep-going --verbose --nospinner --oneshot --quiet-build=n --sync
+1431764402:  === sync
+1431764402: >>> Syncing repository 'gentoo' into '/usr/portage'...
+1431764402: >>> Starting rsync with rsync://[2a01:90:200:10::1a]/gentoo-portage
+1431764460: === Sync completed for gentoo
+1431764493:  *** terminating.
+*/
 _q_static void
 show_sync_history(const char *logfile)
 {
 	FILE *fp;
-	size_t buflen;
-	char *buf, *p, *q;
+	size_t buflen, len;
+	char *buf, *p;
 	time_t t;
 
 	if ((fp = fopen(logfile, "r")) == NULL) {
@@ -318,26 +337,35 @@ show_sync_history(const char *logfile)
 	}
 
 	buf = NULL;
+	/* Just find the finish lines. */
 	while (getline(&buf, &buflen, fp) != -1) {
-		if (strlen(buf) < 35)
-			continue;
-		if (strncmp(buf+12, "=== Sync completed with", 23) != 0)
+		len = strlen(buf);
+		/* This cuts out like ~10% of the log. */
+		if (len < 35)
 			continue;
 
-		if ((p = strchr(buf, '\n')) != NULL)
-			*p = 0;
+		/* Make sure there's a timestamp in here. */
 		if ((p = strchr(buf, ':')) == NULL)
 			continue;
-		*p = 0;
-		q = p+2;
+		p += 2;
+
+		if (strncmp(p, "=== Sync completed ", 19) != 0)
+			continue;
+		p += 19;
+
+		if (buf[len - 1] == '\n')
+			buf[len - 1] = '\0';
 
 		t = (time_t)atol(buf);
 
-		if ((p = strstr(q, "with")) == NULL)
+		if (!strncmp(p, "with ", 5))
+			p += 5;
+		else if (!strncmp(p, "for ", 4))
+			/* This shows just the repo name not the remote host ... */
+			p += 4;
+		else
 			continue;
-		q = p + 5;
-
-		printf("%s >>> %s%s%s\n", chop_ctime(t), GREEN, q, NORM);
+		printf("%s >>> %s%s%s\n", chop_ctime(t), GREEN, p, NORM);
 	}
 
 	free(buf);
@@ -618,13 +646,11 @@ int qlop_main(int argc, char **argv)
 {
 	int i, average = 1;
 	char do_time, do_list, do_unlist, do_sync, do_current, do_human_readable = 0;
-	char *opt_logfile;
-	const char *logfile = QLOP_DEFAULT_LOGFILE;
+	char *logfile = NULL;
 
 	DBG("argc=%d argv[0]=%s argv[1]=%s",
 		argc, argv[0], argc > 1 ? argv[1] : "NULL?");
 
-	opt_logfile = NULL;
 	do_time = do_list = do_unlist = do_sync = do_current = 0;
 
 	while ((i = GETOPT_LONG(QLOP, qlop, "")) != -1) {
@@ -639,15 +665,15 @@ int qlop_main(int argc, char **argv)
 			case 'g': do_time = 1; average = 0; break;
 			case 'H': do_human_readable = 1; break;
 			case 'f':
-				if (opt_logfile) err("Only use -f once");
-				opt_logfile = xstrdup(optarg);
+				if (logfile) err("Only use -f once");
+				logfile = xstrdup(optarg);
 				break;
 		}
 	}
 	if (!do_list && !do_unlist && !do_time && !do_sync && !do_current)
 		qlop_usage(EXIT_FAILURE);
-	if (opt_logfile != NULL)
-		logfile = opt_logfile;
+	if (logfile == NULL)
+		xasprintf(&logfile, "%s/%s", portlogdir, QLOP_DEFAULT_LOGFILE);
 
 	argc -= optind;
 	argv += optind;
@@ -668,7 +694,7 @@ int qlop_main(int argc, char **argv)
 			show_merge_times(argv[i], logfile, average, do_human_readable);
 	}
 
-	if (opt_logfile) free(opt_logfile);
+	free(logfile);
 
 	return EXIT_SUCCESS;
 }
